function [h, Di, beta, c, Q,NorQ] = OOMP_FFT_MixD(f, Dx, tol, No,eex,option,hat,dicl)
%OOMP Optimized Orthogonal Matching Pursuit in 1D dedicated to the dictitonary D_M in [1].
%
% It creates an atomic decomposition of a signal using   the dictitonary D_M in and the OOMP criterion.
% You can choose a  tolerance error or the numebr of atoms 
% Inputs:
%   f       analyzing signal 1D signal
%   Dx       triginometric dictionary of normalized atoms
%   tol     desired distance between f and its approximation. The routine will stop if
%           norm(f'-Dsub*(f*beta)')*sqrt(delta)<tol where delta=1, but can be changed to 
%           1/L, L is number of points  in a sample.
%   No      (optional) maximal number of atoms to choose, if the number of chosen atoms
%           equals to No, routine will stop 
%   option  option=4 for Sin-Cos dicitonary, option=2 for Cos            and option=3 for Sin dictionaries
%   hat    shape for the pulses
%   dicl   dictionary of translated pulses

%
% Outputs:
%   h       the approximated signal
%   Di      indices of 'k' selcted  atoms.
%   beta    biorthogonal atoms to the selected ones. 
%   c       coefficients of the atomic decomposition
%   Q       orthogonalized selected atoms. 
%
% The implementation of OOMP method is based on Gram-Schmidt orthonormalization and 
% adaptive birothogonalization, as proposed in Ref 2 below.
%
%----------------------------------------------------------
%  Laura Rebollo-Neira
%  http://www1.aston.ac.uk/eas/staff/dr-laura-rebollo-neira/
%----------------------------------------------------------
%
%  Reference:
%  1- L.Rebollo-Neira and A. Plastino, "Sparse Represenation of Gravitational Sound", prepint (2017)
%  
%  2-L. Rebollo-Neira and D. Lowe, "Optimized Orthogonal Matching Pursuit Approach", IEEE
%  Signal Processing Letters, Vol(9,4), 137-140, (2002). 
%
%  3-L. Rebollo-Neira, ``Cooperative Pursuit Strategies for sparse approximation by partitioning'',
%  Signal Processing,  Vol(125) 365-375 (2016)
%  
%
%   
name='OOMP_FFT_MixD';  %name of routine
% get inputs and setup parameters
ind=[];
[Lx,Nx]=size(Dx);
nnl=numel(dicl);
Nl=0;
D=Dx;
for ii=1:nnl
Nl=Nl +size(dicl{ii},2);
D=[D dicl{ii}];
end
N=Nx+Nl;
zmax=1;%number of reorthogonalizations
beta=[];
Re=f;
Range=1:N; %  
Di=[];     %  
%delta=1/L;  
delta=1;   

NorQl=zeros(Nl,1);
NorQ=zeros(Nx,1);
if option==4 NorQ=zeros(Nx/2,2);end

numind=length(ind);

%atoms having smaller norm than tol1 are supposed be zero ones
tol1=1e-7; %1e-5
%threshold for coefficients
tol2=1e-10;   %0.0001  %1e-5

%===============================
H=min(No,Lx); %maximal number of function in sub-dictionary
for k=1:H    
  c=zeros(1,N);cc=zeros(1,N);
  if k<=numind 
    [test,q]=ismember(ind(k),Range);
    if test~=1 error('Demanded index %d is out of dictionary',ind(k));end
  else
% select atoms with OOMP and dicitonry D_M 
     [max_c,q,scra]=AtSelTrgLocOOMP(Re,Dx,NorQ,NorQl,eex,option,hat,dicl);
    %stopping criterion (coefficient)
    if max_c<tol2 
      k=k-1;
      fprintf('%s stopped, max(|<f,q>|/||q||)<= tol2=%g.\n',name,tol2);
      break;
    end
  end  
  Di=[Di q];
  if k>1
    for p=1:k-1
     Q(:,k)=D(:,q)-(Q(:,p)'*D(:,q))*Q(:,p); %orthogonalization
    end 
   for zi=1:zmax
    for p=1:k-1
     Q(:,k)=Q(:,k)-(Q(:,p)'*Q(:,k))*Q(:,p); %re-orthogonalization
    end	   
   end
  end
  if k==1 Q(:,k)=D(:,q); end
  nork=norm(Q(:,k)); 
  Q(:,k)=Q(:,k)/nork; %normalization
  % compute biorthogonal vectors beta from 1 to k-1
  if k>1
    beta=beta-Q(:,k)*(D(:,q)'*beta)/nork; 
  end	
  beta(:,k)=Q(:,k)/nork; % kth biorthogonal function
  %
%%%%computes de residum%%%%%   
   Re=Re-f*Q(:,k)*Q(:,k)';
   [NorQ,NorQl]=AuxiTrgLoc(NorQ,NorQl,Q(:,k)',Nx,eex,option,hat,dicl);
   [gl]=find(NorQl(:,1)>=1);
   NorQl(gl,1)=0;
   [gt1]=find(NorQ(:,1)>=1);
   NorQ(gt1,1)=0; %sloppy to delate the already chosen atoms
  if option==4
    [gt2]=find(NorQ(:,2)==1);
     NorQ(gt2,2)=0; %sloppy to delate the already chosen atoms
  end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%
  %stopping criterion (distance)
  if (tol~= 0) & (norm(Re)*sqrt(delta) < tol) break;end;
end
c=f*beta; %coefficients
h=(f-Re);%approximation

%
%Copyright (C) 2016  Laura EBOLLO-NEIRA,  Mathematics Department, Aston University. 
%This program is free software; you can redistribute it and/or modify it under the terms 
%of the GNU General Public License as published by the Free Software Foundation; either 
%version 2 of the License, or (at your option) any later version.
%
%This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
%without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
%See the GNU General Public License for more details.
%
%You should have received a copy of the GNU General Public License along with this program;
%if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
%Boston, MA  02110-1301, USA.
